@@ -7,7 +7,6 @@ import logging
 import traceback
 import time
 
-
 def __append_to_file(output_file_path: str, data):
     with open(output_file_path, 'a') as outfile:
         outfile.write(json.dumps(data))
@@ -21,6 +20,7 @@ def __clean_str_for_json(text: str):
 
 def ask_questions(configs : list, questions : list, output_file_path : str, verbose: bool = False, model : str = "gpt-3.5-turbo", system_text : str | None = None, temperature : float | None = 1, max_tokens : int | None = 2048) -> None:
     question_queue = queue.Queue()
+    save_queue = queue.Queue()
 
     def loader_worker():
         if verbose:
@@ -50,8 +50,14 @@ def ask_questions(configs : list, questions : list, output_file_path : str, verb
 
         if question_queue.empty():
             print("[sleepyask] All questions exhausted")
-            print("""
-***     DONE ASKING ALL QUESTIONS       ***""")
+            print("\n***     DONE ASKING ALL QUESTIONS       ***")
+
+    def saver_worker():
+        while True:
+            if save_queue.empty(): continue
+            to_add = save_queue.get()
+            __append_to_file(output_file_path, to_add)
+            save_queue.task_done()
 
     def asker_worker(index, config):
         import openai
@@ -60,11 +66,11 @@ def ask_questions(configs : list, questions : list, output_file_path : str, verb
 
         succeed = True
         while succeed:
+            if question_queue.empty(): succeed = False
             question = question_queue.get()
             message = ''
             if verbose:
                 print(f"[sleepyask {index}] Asking:", question["question"])
-            # logging.disable(logging.ERROR)
             try:
                 messages = [
                     {"role": "user", "content": question["question"]},
@@ -80,6 +86,7 @@ def ask_questions(configs : list, questions : list, output_file_path : str, verb
                     model=model,
                     messages=messages
                 )
+
                 actual_model = message["model"]
                 usage = message["usage"]
                 message = message["choices"][0]["message"]["content"]
@@ -90,15 +97,15 @@ def ask_questions(configs : list, questions : list, output_file_path : str, verb
                 dt_string = datetime.now().strftime("%d/%m/%Y %H:%M:%S")
                 row_to_append = {"model": actual_model, "temperature": temperature, "max_tokens": max_tokens, "date_time": dt_string, "question_number":
                                  question["question_number"], "question": question["question"], "system_text": str(system_text), "response": __clean_str_for_json(message), **usage}
-                __append_to_file(output_file_path, row_to_append)
+                save_queue.put(row_to_append)
 
+            except KeyboardInterrupt:
+                succeed = False
+                break
             except openai.error.AuthenticationError:
                 logging.error(traceback.format_exc())
                 question_queue.put(question)
                 time.sleep(120)
-            except:
-                question_queue.put(question)
-                time.sleep(120)
 
             question_queue.task_done()
 
@@ -110,11 +117,16 @@ def ask_questions(configs : list, questions : list, output_file_path : str, verb
             thread_refs.append(t)
             t.start()
 
-    t = threading.Thread(target=loader_worker, daemon=True)
-    thread_refs.append(t)
-    t.start()
+    loader = threading.Thread(target=loader_worker, daemon=True)
+    saver = threading.Thread(target=saver_worker, daemon=True)
+    thread_refs.append(loader)
+    thread_refs.append(saver)
+    
+    loader.start()
+    saver.start()
 
     question_queue.join()
+    save_queue.join()
 
     for t in thread_refs:
         t.join()
